/**
 * buffershadow.java
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.*;

import gl4java.GLContext;
import gl4java.awt.GLCanvas;
import gl4java.utils.glut.*;
import gl4java.utils.textures.*;

/**
 * buffershadow.java is a program to demonstrate features like 
 * shadows, texture and other means to make an OpenGL scene
 * realistic. To rotate the scene, use the arrow-keys
 * LEFT AND RIGHT ARROW: Rotate scene around y-axis
 * UP AND DOWN ARROW: Rotate scene around x-axis
 *
 * @author <a href="mailto:oyvindli@odin.hiof.no">yvind Liland</a>
 * @version 1.0
 */

public class buffershadow extends JFrame {
    JPanel panel;
 
    myGLCanvas canvas = null;

    //Gir hver farge et integer for bruk i display list
    private static int BLUE = 1;
    private static int RED = 2;	
    private static int WHITE = 3;

    //Int for  avgjre hva slags textur som skal brukes p cube
    private static int DRAWER_FRONT = 0;
    private static int RED_TEXTURE = 1; 

    //Int for  avgjre hva slags plan som skal tegnes
    private static final int FLOOR = 1;
    private static final int BACK_WALL = 2;
    private static final int LEFT_WALL = 3;

    //Array for  lagre skygge
    private float shadow_floor[] = new float[16];
    private float shadow_back_wall[] = new float[16];
    private float shadow_left_wall[] = new float[16];
       
    //Windows events
    WindowHandler wHandler;
    KeyHandler kHandler; 

    // rotation angles, set in mousedragged
    // used in display
    private static float rotx=0.0f;
    private static float roty=0.0f;

    //Constructor
    /**
     * Creates a new <code>buffershadow</code> instance.
     *
     */
    public buffershadow () {
	
	panel = (JPanel) this.getContentPane();	
	panel.setLayout(new BorderLayout());
	this.setSize(new Dimension(600,600));
	this.setTitle("BufferShadow");
	
	//gl4java initiering
	Dimension d = getSize();
	canvas = new myGLCanvas(d.width, d.height);
	panel.add("Center", canvas);

	wHandler = new WindowHandler();
	kHandler = new KeyHandler();
	this.addWindowListener(wHandler);
	this.addKeyListener(kHandler);

	pack();
	show();
    }

    //Main method
    /**
     * <code>main</code>
     *
     * @param args a <code>String[]</code> value
     */
    public static void main(String[]args){
	new buffershadow();
    }

    //---------------------------------------------------------    

    /**
     * <code>WindowHandler</code>
     * Inner class for window events, interface WindowListener
     */
    class WindowHandler implements WindowListener {
	public void windowOpened(WindowEvent e){}
	public void windowClosing(WindowEvent e){
	    System.exit(0);
	}
	public void windowClosed(WindowEvent e){}
	public void windowIconified(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowActivated(WindowEvent e){}
	public void windowDeactivated(WindowEvent e){}
    }
    //-------------------------------------------------------------------
    
    /**
     * <code>KeyHandler</code>
     * Inner class for key events, interface KeyListener
     */
    class KeyHandler implements KeyListener {
	public void keyPressed(KeyEvent e){   
	    //KeyHandler for  kunne utforske scenen. Kan rotere hele scenen om x-aksen
	    //og y-aksen ved hjelp av piltastene
	    switch (e.getKeyCode())
		{
		case KeyEvent.VK_LEFT: 
		    if (roty == 0) {
			roty = 360.0f;
		    }
		    roty -= 10.0f;    
		    canvas.repaint();		    
		    break;
		case KeyEvent.VK_RIGHT:
		    if (roty == 360) {
			roty = 0.0f;
		    }
		    roty += 10.0f;    
		    canvas.repaint();
		    break;		  
		case KeyEvent.VK_UP: 
		    if (rotx == 0) {
			rotx = 360.0f;
		    }
		    rotx -= 10.0f;    
		    canvas.repaint();		    
		    break;
		case KeyEvent.VK_DOWN:
		    if (rotx == 360) {
			rotx = 0.0f;
		    }
		    rotx += 10.0f;    
		    canvas.repaint();
		    break;		  
		} 
	}
	public void keyReleased(KeyEvent e){}	
	public void keyTyped(KeyEvent e){}
    }
    //--------------------------------------------------------------------------------------------------------
    /**
     * <code>myGLCanvas</code>
     * Inner class myGLCanvas extends GLCanvas
     */
    private class myGLCanvas extends GLCanvas {
	GLUTFunc glut = null;

	//Original lysposisjon
	float light_position[] = { 30.0f, 20.0f, 80.0f, 1.0f };
	
	//Setter koordinatene for gulv og vegger.
	float floor[][] = 
	  {  
	      {-30.0f,-5.0f,-30.0f},
	      {-30.0f,-5.0f,50.0f},
	      {30.0f,-5.0f,50.0f},
	      {30.0f,-5.0f,-30.0f}
	  };

	float back_wall[][] = 
	  {
	      {-30.0f, -5.0f, -30.0f},
	      {-30.0f, 25.0f, -30.0f},
	      {30.0f, 25.0f, -30.0f},
	      {30.0f, -5.0f,-30.0f}
	  };

	float left_wall[][] = 
	  {
	      {-30.0f,-5.0f,50.0f},
	      {-30.0f,25.0f,50.0f},
	      {-30.0f,25.0f,-30.0f},
	      {-30.0f,-5.0f,-30.0f}
	  };

	//Normalene til gulv og vegger
	float normals[][] =
	  {
	      {1.0f,0.0f,0.0f},  //Left wall
	      {-1.0f,0.0f,0.0f}, //Right wall
	      {0.0f,0.0f,1.0f},  //Back wall
	      {0.0f,1.0f,0.0f}   //Floor
	  };

	//Storage for one texture
	int texture[][] = new int[3][1];
	int tex_id [] = {1,2,3};

	//Name on the file to be loaded as a texture
	String fileName_1 = "data/wood_light.png";
	String fileName_2 = "data/kommode_front.png";
	String fileName_3 = "data/kommode_texture.png";

	String fileName[] = {fileName_1, fileName_2, fileName_3};

	//Material properties
	float blue_ambient[] = { 0.1f, 0.1f, 0.6f, 1.0f };
	float blue_diffuse[] = { 0.2f, 0.2f, 0.8f, 1.0f };
	float red_ambient[] = { 0.5f, 0.0f, 0.0f, 1.0f };
	float red_diffuse[] = { 0.7f, 0.2f, 0.2f, 1.0f };
	float no_specular[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	float white[] = {1.0f,1.0f,1.0f,1.0f};
	float black[] = {0.0f,0.0f,0.0f,0.0f};

	float high_shininess[] = {100.0f};
	float no_shininess[] = {0.0f};
	float low_shininess[] = {10.0f};

	/**
	 * Creates a new <code>myGLCanvas</code> instance.
	 *
	 * @param w an <code>int</code> value
	 * @param h an <code>int</code> value
	 */
	public myGLCanvas(int w, int h) {
	    super(w, h);
	}
	

	/**
	 * <code>preInit</code>
	 * Kalles fr myGLCanvas blir skapt
	 */
	public void preInit() {
	    doubleBuffer = true;
	    stereoView = false;
	    accumSize = 8;
	    stencilBits = 3;
	}
    
	
	/**
	 * <code>init</code>
	 * Kalles nr programmet starter og tar seg av engans initiering.
	 * Typisk lyssetting, glattemodus, bakgrunnsfarge
	 */
	public void init() {
	    reshape (getSize().width, getSize().height);
	    glut = new GLUTFuncLightImpl(gl,glu);  

	    //Plasserer lyset
	    gl.glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	    gl.glClearColor(0.1f,0.1f,0.7f,0.0f);

	    //NB: Lage lister over materialproperties
	    gl.glNewList(BLUE, GL_COMPILE);
	      gl.glMaterialfv(GL_FRONT, GL_AMBIENT, blue_ambient);
	      gl.glMaterialfv(GL_FRONT, GL_DIFFUSE, blue_diffuse);
	      gl.glMaterialfv(GL_FRONT, GL_SPECULAR, white);
	      gl.glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
	    gl.glEndList();	    

	    gl.glNewList(RED, GL_COMPILE);
	      gl.glMaterialfv(GL_FRONT, GL_AMBIENT, red_ambient);
	      gl.glMaterialfv(GL_FRONT, GL_DIFFUSE, red_diffuse);
	      gl.glMaterialfv(GL_FRONT, GL_SPECULAR, no_specular);
	      gl.glMaterialfv(GL_FRONT, GL_SHININESS, no_shininess);
	    gl.glEndList();

	    gl.glNewList(WHITE, GL_COMPILE);
	      gl.glMaterialfv(GL_FRONT, GL_AMBIENT, white);
	      gl.glMaterialfv(GL_FRONT, GL_DIFFUSE, white);
	      gl.glMaterialfv(GL_FRONT, GL_SPECULAR, white);
	      gl.glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
	    gl.glEndList();

	    //Laster teksturer for bruk p gulv og mbler
	    loadTexture();

	    //Finner planet for gulv basert p tre kjente punkter
	    float plane_floor[] = calculatePlane(floor[1],floor[2],floor[3]); 
	    //Lager shadowMatrix for gulv
	    shadow_floor = shadowMatrix(plane_floor,light_position);

	    //Finner planet for back_wall
	    float plane_back_wall[] = calculatePlane(back_wall[1],back_wall[2],back_wall[3]);
	    //Lager shadowMatrix for back_wall
	    shadow_back_wall = shadowMatrix(plane_back_wall,light_position);

	    //Finner planet for left_wall
	    float plane_left_wall[] = calculatePlane(left_wall[1],left_wall[2],left_wall[3]);
	    //Lager shadowMatrix for left_wall
	    shadow_left_wall = shadowMatrix(plane_left_wall, light_position);

	    gl.glShadeModel(GL_SMOOTH);
	    gl.glEnable(GL_LIGHTING);
 	    gl.glEnable(GL_LIGHT0);
	    gl.glEnable(GL_DEPTH_TEST);
	    gl.glEnable(GL_NORMALIZE);
	}
	
	
	/**
	 * <code>loadTexture</code>
	 * Textureloader. Legger teksturer inn i array texture for senere bruk
	 */
	public void loadTexture(){
	    TextureLoader texLoader = new PngTextureLoader(gl,glu);

	    if (texLoader.isOk()) {
		for (int i = 0; i < fileName.length; i ++) {
		    texLoader.readTexture(fileName[i]);

		    gl.glGenTextures(tex_id[i],texture[i]);
		    gl.glBindTexture(GL_TEXTURE_2D,texture[i][0]);

		    gl.glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		    gl.glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

		    gl.glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		    gl.glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		
		    gl.glTexImage2D(GL_TEXTURE_2D, 0, 3, texLoader.getImageWidth(),
				texLoader.getImageHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE, texLoader.getTexture());	
		}
	    }
	}
	
	/**
	 * <code>display</code>
	 * Kalles fr myGLCanvas blir skapt
	 */
	public void display() {
	    if (glj.gljMakeCurrent() == false)
		return;

	    gl.glMatrixMode(GL_MODELVIEW);
	    gl.glLoadIdentity();   
	    
 	    glu.gluLookAt(0.0f,20.0f,90.0f,// eye
 			  0.0f,0.0f,0.0f, // looking at			
			  0.0f,1.0f,0.0f);// is up	    
	    
	    // rotate around x-axis
	    gl.glRotatef(rotx,1.0f,0.0f,0.0f);
	    // rotate around y-axis
	    gl.glRotatef(roty,0.0f,1.0f,0.0f);
	    
	    //----------------------------------------------------------
	    //Fjern kommenteringen p denne delen for  oppn myke kanter p skygge
	    //Jittering code
	    /*
     	    float dx,dy,dz;
     	    dx = -0.2f;
     	    dy = 0.0f;
     	    dz = -0.2f;

     	    gl.glClear(GL_ACCUM_BUFFER_BIT);
     	    for (int i = 0; i<4; i++){
     		dz+=0.1f;
     		for (int j = 0; j<4; j++){
     		    dx+=0.1f;
     		    drawScene(dx,dy,dz);
     		    gl.glAccum(GL_ACCUM, 1.0f/16.0f);
     		}  		
    	    }
   	    gl.glAccum(GL_RETURN, 1.0f);
 	    */
	    //----------------------------------------------------------

	    //Kommenter ut drawScene under hvis du bruker jitter-kode
	    drawScene(0.0f,0.0f,0.0f);

	    glj.gljSwap();
	    glj.gljFree();
	} 

	/**
	 * <code>renderPlane</code>
	 * 
	 * @param type_of_plane an <code>int</code> value
	 */
	public void renderPlane(int type_of_plane) {
	    //Skrur p stencil test
	    gl.glEnable(GL_STENCIL_TEST);
	    //Bruker type_of_plane som unik identifikator for hver stencil buffer
	    gl.glStencilFunc(GL_ALWAYS, type_of_plane, ~0);
	    gl.glStencilOp (GL_KEEP, GL_KEEP, GL_REPLACE);

	    switch (type_of_plane) {
	    case FLOOR:
		gl.glCallList(WHITE);
		drawFloor(floor,normals[3]); 
		break;
		
	    case BACK_WALL:
		gl.glCallList(WHITE);
		drawWall(back_wall,normals[2]);
		break;
		
	    case LEFT_WALL:		
		gl.glCallList(WHITE);
		drawWall(left_wall,normals[0]);
		break;
		
       	    default:
		System.err.println("Error in method renderPlane(int,int)");
		break;
	    } // end of switch (type_of_plane)
	    
	    gl.glDisable(GL_STENCIL_TEST);

	}

	/**
	 * <code>shadowMode</code>
	 *
	 * @param type_of_plane an <code>int</code> value
	 */
	public void shadowMode(int type_of_plane) {
	    //Tegner skygge av objekter p gulv
	    gl.glDisable(GL_DEPTH_TEST); 
  	    gl.glEnable(GL_STENCIL_TEST);
  	    gl.glStencilFunc(GL_EQUAL, type_of_plane, ~0);
  	    gl.glStencilOp(GL_KEEP,GL_KEEP,GL_ZERO);

  	    gl.glEnable(GL_BLEND);
  	    gl.glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	    gl.glDisable(GL_LIGHTING);
  	    gl.glColor4f(0.0f,0.0f,0.0f,0.5f);  
	}

	/**
	 * <code>restoreBasicMode</code>
	 *
	 */
	public void restoreBasicMode(){
	    //Restore basic modes
  	    gl.glDisable(GL_BLEND);
  	    gl.glDisable(GL_STENCIL_TEST);
  	    gl.glEnable(GL_DEPTH_TEST);
 	    gl.glEnable(GL_LIGHTING); 
	}
	

	
	/**
	 * <code>drawScene</code>
	 * Metode som tegner hele scenen. 
	 *
	 * @param dx a <code>float</code> value
	 * @param dy a <code>float</code> value
	 * @param dz a <code>float</code> value
	 */
	public void drawScene(float dx, float dy, float dz) {
	    gl.glClear(GL_COLOR_BUFFER_BIT |
		       GL_DEPTH_BUFFER_BIT |
		       GL_STENCIL_BUFFER_BIT);

	    //Rendrer gulv og vegger i stencil buffer og tegner disse
	    //Gulv
	    gl.glPushMatrix();
	    renderPlane(FLOOR);
	    gl.glPopMatrix();

	    //Tegner bak-vegg
	    gl.glPushMatrix();
	    renderPlane(BACK_WALL);
	    gl.glPopMatrix();

	    //Tegner venstre-vegg
	    gl.glPushMatrix();
	    renderPlane(LEFT_WALL);
	    gl.glPopMatrix();
	    //-----------------------------------------------------------
	    //Tegner skygge av objekter p gulv
	    shadowMode(FLOOR);

	    //Tegner skygge av kommode p gulv
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_floor);
  	    gl.glTranslatef(dx,dy,dz);
  	    drawDrawer();
  	    gl.glPopMatrix();

	    //Tegner skygge av tekanne p gulv
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_floor);
  	    gl.glTranslatef(dx,dy,dz);
	    drawTeapot();
  	    gl.glPopMatrix();

	    //Tegner skygge av stol 1 p gulv
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_floor);
  	    gl.glTranslatef(dx,dy,dz);
	    drawChair(35f,0f,10f,270f);
  	    gl.glPopMatrix();

	    //Tegner skygge av stol 2 p gulv
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_floor);
  	    gl.glTranslatef(dx,dy,dz);
	    drawChair(3f,0f,-7f,0f);
  	    gl.glPopMatrix();

	    //Tegner skygge av bord p gulv
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_floor);
  	    gl.glTranslatef(dx,dy,dz);
	    drawTable();
  	    gl.glPopMatrix();

	    restoreBasicMode();
	    //--------------------------------------------------------------
	    //Tegner skygge av objekter p bak-vegg
	    shadowMode(BACK_WALL);

	    //Tegner skygge av kommode p bak-vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_back_wall);
  	    gl.glTranslatef(dx,dy,dz);
  	    drawDrawer();
  	    gl.glPopMatrix();

	    //Tegner skygge av tekanne p bak-vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_back_wall);
  	    gl.glTranslatef(dx,dy,dz);
	    drawTeapot();
  	    gl.glPopMatrix();

	    //Tegner skygge av stol 1 p bak-vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_back_wall);
  	    gl.glTranslatef(dx,dy,dz);
	    drawChair(35f,0f,10f,270f);
  	    gl.glPopMatrix();

	    //Tegner skygge av stol 2 p bak-vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_back_wall);
  	    gl.glTranslatef(dx,dy,dz);
	    drawChair(3f,0f,-7f,0f);
  	    gl.glPopMatrix();

	    //Tegner skygge av bord p bak-vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_back_wall);
  	    gl.glTranslatef(dx,dy,dz);
	    drawTable();
  	    gl.glPopMatrix();

	    restoreBasicMode();
	    //--------------------------------------------------------------
	    //Tegner skygge av objekter p venstre vegg
	    shadowMode(LEFT_WALL);

	    //Tegner skygge av kommode p venstre vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_left_wall);
  	    gl.glTranslatef(dx,dy,dz);
  	    drawDrawer();
  	    gl.glPopMatrix();

	    //Tegner skygge av tekanne p venstre vegg
  	    gl.glPushMatrix();
  	    gl.glMultMatrixf(shadow_left_wall);
  	    gl.glTranslatef(dx,dy,dz);
	    drawTeapot();
  	    gl.glPopMatrix();

	    restoreBasicMode();
	    //--------------------------------------------------------------
	    //Tegner kommode
	    gl.glPushMatrix();
	    drawDrawer();    
	    gl.glPopMatrix();

	    //Tegner tekanne	    
  	    gl.glPushMatrix();
  	    gl.glCallList(BLUE);
  	    drawTeapot();
	    gl.glPopMatrix();
	    
	    //Tegner stol 1
	    gl.glPushMatrix();
	    gl.glCallList(WHITE);
	    drawChair(35f,0f,10f,270f);
	    gl.glPopMatrix();

	    //Tegner stol 2	    
	    gl.glPushMatrix();
	    gl.glCallList(WHITE);
	    drawChair(3f,0f,-7f,0f);
	    gl.glPopMatrix();

	    //Tegner bord
	    gl.glPushMatrix();
	    gl.glCallList(WHITE);
	    drawTable();
	    gl.glPopMatrix();
	    
	}

	
	/**
	 * <code>calculatePlane</code>
	 * Finner planet p formen ax + by + cx + d, basert p tre kjente punkter i planet
         * p0, p1, p2. a,b,c er normalen til planet, 
	 *
	 * @param p1[] a <code>float</code> value
	 * @param p2[] a <code>float</code> value
	 * @param p3[] a <code>float</code> value
	 * @return a <code>float[]</code> value
	 */
	public float [] calculatePlane(float p1[], float p2[], float p3[]) {
	    //Array for planlikningen
	    float plane[] = new float[4];
	    
	    //Gitt to vektorer (tre punkter) i planet kan normalen regnes ut
	    //Vi vil ha aboluttverdier
 	    plane[0] = Math.abs(((p2[1]-p1[1])*(p3[2]-p1[2]))-((p2[2]-p1[2])*(p3[1]-p1[1])));
 	    plane[1] = Math.abs(((p2[2]-p1[2])*(p3[0]-p1[0]))-((p2[0]-p1[0])*(p3[2]-p1[2])));
 	    plane[2] = Math.abs(((p2[0]-p1[0])*(p3[1]-p1[1]))-((p2[1]-p1[1])*(p3[0]-p1[0])));
	    plane[3] = -(plane[0]*p1[0] + plane[1]*p1[1] + plane[2]*p1[2]);    
	    

	    return plane;
	}

	/**
	 * <code>drawTable</code>
	 *
	 * Metode for  tegne et bord
	 */
	public void drawTable () {
	    gl.glPushMatrix();
	    //En slags "global" translate som flytter hele objektet
	    //Setter den her slik at vi ikke m kjre translate bde p
	    //Objekt og skyggen til objektet p de forskjellige planene
	    gl.glTranslatef(2.0f,0.0f,3.0f);

	    //Tegner bordplate
	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,3.0f,20.0f);
	    drawCube(0.5f,10,10,RED_TEXTURE);
 	    gl.glPopMatrix();

	    //Tegner bordbein
	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,-5.0f,20.0f);
	    drawCube(8,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(14.0f,-5.0f,20.0f);
	    drawCube(8,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(14.0f,-5.0f,11.0f);
	    drawCube(8,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,-5.0f,11.0f);
	    drawCube(8,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPopMatrix();
	}
	
	/**
	 * <code>drawChair</code>
	 *
	 * Metode for  tegne en stol. Parametere bestemmer plassering i x,y og z 
	 * retning. Parameteret degrees bestemmer hvor mye om y-aksen stolen skal roteres
	 *
	 * @param x a <code>float</code> value
	 * @param y a <code>float</code> value
	 * @param z a <code>float</code> value
	 * @param degrees a <code>float</code> value
	 */
	public void drawChair (float x, float y, float z, float degrees) {
	    gl.glPushMatrix();
	    //En slags "global" translate som flytter hele objektet
	    //Setter den her slik at vi ikke m kjre translate bde p
	    //Objekt og skyggen til objektet p de forskjellige planene
	    gl.glTranslatef(x,y,z);
	    gl.glRotatef(degrees,0.0f,1.0f,0.0f);

	    //Tegner stolsete
	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,0.0f,20.0f);
	    drawCube(0.5f,7,7,RED_TEXTURE);
 	    gl.glPopMatrix();

	    //Tegner stolbein
	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,-5.0f,20.0f);
	    drawCube(5,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(11.0f,-5.0f,20.0f);
	    drawCube(5,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(11.0f,-5.0f,14.0f);
	    drawCube(5,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,-5.0f,14.0f);
	    drawCube(5,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    //Tegner spiler i stolrygg
	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,0.5f,14.0f);
	    drawCube(6,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(11.5f,0.5f,14.0f);
	    drawCube(6,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(7.0f,0.5f,14.0f);
	    drawCube(6,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPushMatrix();
	    gl.glTranslatef(9.5f,0.5f,14.0f);
	    drawCube(6,0.5f,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    //Tegner platen til rygglenet
	    gl.glPushMatrix();
	    gl.glTranslatef(5.0f,6.5f,14.0f);
	    drawCube(2,7,0.5f,RED_TEXTURE);
	    gl.glPopMatrix();

	    gl.glPopMatrix();
	}


	
	/**
	 * Describe <code>shadowMatrix</code> method here.
	 *
	 * Lager en skyggematrise ut i fra planlikning og lysposisjon
	 *
	 * @param plane[] a <code>float</code> value
	 * @param light_pos[] a <code>float</code> value
	 * @return a <code>float[]</code> value
	 */
	public float[] shadowMatrix( float plane[], float light_pos[]) {
	    float shadow_mat[] = new float[16];
	    float dot;

	    //Finner dot produktet mellom lysvektor og planets normal
	    dot = plane[0] * light_pos[0] + 
		  plane[1] * light_pos[1] +
		  plane[2] * light_pos[2] +
		  plane[3] * light_pos[3];

	    shadow_mat[0] = dot - light_pos[0] * plane[0];
	    shadow_mat[4] = -light_pos[0] * plane[1];
            shadow_mat[8]  = -light_pos[0] * plane[2];
            shadow_mat[12] = -light_pos[0] * plane[3];
            
            shadow_mat[1]  = -light_pos[1] * plane[0];
            shadow_mat[5]  = dot - light_pos[1] * plane[1];
            shadow_mat[9]  = -light_pos[1] * plane[2];
            shadow_mat[13] = -light_pos[1] * plane[3];
            
            shadow_mat[2]  = -light_pos[2] * plane[0];
            shadow_mat[6]  = -light_pos[2] * plane[1];
            shadow_mat[10] = dot - light_pos[2] * plane[2];
            shadow_mat[14] = -light_pos[2] * plane[3];
            
            shadow_mat[3]  = -light_pos[3] * plane[0];
            shadow_mat[7]  = -light_pos[3] * plane[1];
            shadow_mat[11] = -light_pos[3] * plane[2];
            shadow_mat[15] = dot - light_pos[3] * plane[3];

	    return shadow_mat;
	}
	
	/**
	 * <code>drawTeapot</code>
	 *
	 * Tegner tekanne
	 */
	public void drawTeapot(){
	    gl.glPushMatrix();
	    //En slags "global" translate som flytter hele objektet
	    //Setter den her slik at vi ikke m kjre translate bde p
	    //Objekt og skyggen til objektet p de forskjellige planene
	      gl.glTranslatef(-10.0f,8.0f,5.0f);

	      gl.glPushMatrix();
  	        glut.glutSolidTeapot(3);
  	      gl.glPopMatrix();
	    gl.glPopMatrix();
	}

	
 	/**
	 * <code>drawDrawer</code>
	 *
	 * Tegner kommode	
	 */
	public void drawDrawer() {
	    gl.glPushMatrix();
	    //En slags "global" translate som flytter hele objektet
	    //Setter den her slik at vi ikke m kjre translate bde p
	    //Objekt og skyggen til objektet p de forskjellige planene
	      gl.glTranslatef(-18.0f,0.0f,10.0f);

	      gl.glPushMatrix();
	        gl.glCallList(WHITE);
	        gl.glTranslatef(0.0f,-4.0f,0.0f);
	        drawCube(10,16,10, DRAWER_FRONT);
	      gl.glPopMatrix();	    

	      gl.glPushMatrix();
	        gl.glCallList(WHITE);
	        gl.glTranslatef(0.0f,-5.0f,0.0f);
	        drawCube(1,1,1, RED_TEXTURE);
	      gl.glPopMatrix();

	      gl.glPushMatrix();
	        gl.glCallList(WHITE);
	        gl.glTranslatef(0.0f,-5.0f,-9.0f);
	        drawCube(1,1,1, RED_TEXTURE);
	      gl.glPopMatrix();

	      gl.glPushMatrix();
	        gl.glCallList(WHITE);
	        gl.glTranslatef(15.0f,-5.0f,0.0f);
	        drawCube(1,1,1, RED_TEXTURE);
	      gl.glPopMatrix();

	      gl.glPushMatrix();
	        gl.glCallList(WHITE);
	        gl.glTranslatef(15.0f,-5.0f,-9.0f);
	        drawCube(1,1,1, RED_TEXTURE);
	      gl.glPopMatrix();

	    gl.glPopMatrix();

	}


	/**
	 * <code>enableTexture</code>
	 *
	 * Metode for  enable og binde textur
	 *
	 * @param tex an <code>int</code> value
	 */
	public void enableTexture (int tex) {
	    //Aktiviserer tekstur og henter denne fra texture-array.
  	    gl.glEnable(GL_TEXTURE_2D);
 	    gl.glBindTexture(GL_TEXTURE_2D, texture[tex][0]);
	    
	    //Tekstur skal overskygge alle materialegenskaper som gulvet eventuelt m ha
 	    gl.glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	}

	/**
	 * <code>disableTexture</code> 
	 *
	 * Skrur av texture
	 */
	public void disableTexture() {
	    //Skrur av teksture slik at ingen andre objekter i scenen skal bli tegnet med tekstur
	    gl.glDisable(GL_TEXTURE_2D);
	}

	
	/**
	 * <code>drawCube</code>
	 *
	 * Metoden tegner en kube med gitt parametre height,width og depth. Plasserer
	 * kuben med venstre hjrne i origo (verdens koordinatene).
	 *
	 * @param height a <code>float</code> value
	 * @param width a <code>float</code> value
	 * @param depth a <code>float</code> value
	 * @param type_of_cube an <code>int</code> value
	 */
	public void drawCube(float height, float width, float depth, int type_of_cube) {

	    if (type_of_cube == DRAWER_FRONT) {
		enableTexture(1); 
	    }else if (type_of_cube == RED_TEXTURE) {
		enableTexture(2);
	    }	    

	    //Tegner forsiden av kube
	    gl.glBegin(GL_QUADS);
	      //Normalen peker langs positiv z-akse
	      gl.glNormal3f(0.0f,0.0f,1.0f);
	      gl.glTexCoord2f(0.0f,0.0f);//Bottom left-corner
	      gl.glVertex3f(0.0f, 0.0f, 0.0f);
	      gl.glTexCoord2f(0.0f,1.0f);//Top left-corner
	      gl.glVertex3f(0.0f, height, 0.0f);
	      gl.glTexCoord2f(1.0f,1.0f);//Top right-corner
	      gl.glVertex3f(width, height, 0.0f);
	      gl.glTexCoord2f(1.0f,0.0f);//Bottom right-corner
	      gl.glVertex3f(width, 0.0f, 0.0f);
	    gl.glEnd();

	    if (type_of_cube == DRAWER_FRONT) {
		disableTexture(); 
		enableTexture(2);
	    } 	    

	    //Tegner baksiden av kube
	    gl.glBegin(GL_QUADS);
	      //Normalen peker langs negativ z-akse
	      gl.glNormal3f(0.0f,0.0f,-1.0f);
	      gl.glTexCoord2f(0.0f,0.0f);//Bottom left-corner
	      gl.glVertex3f(0.0f, 0.0f, -depth);
	      gl.glTexCoord2f(0.0f,1.0f);//Top left-corner
	      gl.glVertex3f(0.0f, height, -depth);
	      gl.glTexCoord2f(1.0f,1.0f);//Top right-corner
	      gl.glVertex3f(width, height, -depth);
	      gl.glTexCoord2f(1.0f,0.0f);//Bottom right-corner
	      gl.glVertex3f(width, 0.0f, -depth);
	    gl.glEnd();	   

	    //Tegner toppen av kube
	    gl.glBegin(GL_QUADS);
	      //Normalen peker langs positiv y-akse
	      gl.glNormal3f(0.0f,1.0f,0.0f);
	      gl.glTexCoord2f(0.0f,0.0f);//Bottom left-corner
	      gl.glVertex3f(0.0f, height, 0.0f);
	      gl.glTexCoord2f(0.0f,1.0f);//Top left-corner
	      gl.glVertex3f(0.0f, height, -depth);
	      gl.glTexCoord2f(1.0f,1.0f);//Top right-corner
	      gl.glVertex3f(width, height, -depth);
	      gl.glTexCoord2f(1.0f,0.0f);//Bottom right-corner
	      gl.glVertex3f(width, height, 0.0f);
	    gl.glEnd();	   

	    //Tegner venstre side av kube
	    gl.glBegin(GL_QUADS);
	      //Normalen peker langs negativ x-akse
	      gl.glNormal3f(-1.0f,0.0f,0.0f);
	      gl.glTexCoord2f(1.0f,0.0f);//Bottom right-corner
	      gl.glVertex3f(0.0f, 0.0f, 0.0f);
	      gl.glTexCoord2f(1.0f,1.0f);//Top right-corner
	      gl.glVertex3f(0.0f, height, 0.0f);
	      gl.glTexCoord2f(0.0f,1.0f);//Top left-corner
	      gl.glVertex3f(0.0f, height, -depth);
	      gl.glTexCoord2f(0.0f,0.0f);//Bottom left-corner
	      gl.glVertex3f(0.0f, 0.0f, -depth);
	    gl.glEnd();	   

	    //Tegner hyre side av kube
	    gl.glBegin(GL_QUADS);
	      //Normalen peker langs positiv x-akse
	      gl.glNormal3f(1.0f,0.0f,0.0f);
	      gl.glTexCoord2f(1.0f,0.0f);//Bottom right-corner
	      gl.glVertex3f(width, 0.0f, 0.0f);
	      gl.glTexCoord2f(1.0f,1.0f);//Top right-corner
	      gl.glVertex3f(width, height, 0.0f);
	      gl.glTexCoord2f(0.0f,1.0f);//Top left-corner
	      gl.glVertex3f(width, height, -depth);
	      gl.glTexCoord2f(0.0f,0.0f);//Bottom left-corner
	      gl.glVertex3f(width, 0.0f, -depth);
	    gl.glEnd();	   

	    //Skrur av texture	      
	    disableTexture();

	    //Tegner bunnen av kube
	    gl.glBegin(GL_QUADS);
	      //Normalen peker langs negativ y-akse
	      gl.glNormal3f(0.0f,-1.0f,0.0f);
	      gl.glVertex3f(0.0f, 0.0f, 0.0f);
	      gl.glVertex3f(0.0f, 0.0f, -depth);
	      gl.glVertex3f(width, 0.0f, -depth);
	      gl.glVertex3f(width, 0.0f, 0.0f);
	    gl.glEnd();	 

	}

	
	/**
	 * <code>drawFloor</code>
	 *
	 * Metode for  tegne gulvet
	 *
	 * @param coo[][] a <code>float</code> value
	 * @param normal[] a <code>float</code> value
	 */
	public void drawFloor(float coo[][],float normal[]){
	    //Array som inneholder hjrnene til texturen. Hjrnene skal mappes slik at 
	    //de ligger p korensponderende hjrner p gulvet
	    float corner[][] = 
		{
		    {0.0f,1.0f},//Top-left corner
		    {0.0f,0.0f},//Bottom-left corner
		    {1.0f,0.0f},//Bottom-right corner
		    {1.0f,1.0f}//Top-right corner
		};
	    
	    //Aktiviserer tekstur og henter denne fra texture-array.
	    gl.glEnable(GL_TEXTURE_2D);
	    gl.glBindTexture(GL_TEXTURE_2D, texture[0][0]);
	    //Vil bevare material properties til gulvet for at lyset skal kunne reflekteres
	    //naturlig, gjr dette med parameter GL_MODULATE
	    gl.glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	    gl.glBegin(GL_QUADS);
	      //Setter normalen (Peker opp langs positiv y)
	      gl.glNormal3f(normal[0],normal[1],normal[2]);
	      for (int i = 0; i < coo.length; i++) {
		  //Binder tekstur til gulvet og tegner
		  gl.glTexCoord2f(corner[i][0],corner[i][1]);
		  gl.glVertex3f(coo[i][0],coo[i][1],coo[i][2]);	    
	      }
	    gl.glEnd();
	    
	    //Skrur av teksture slik at ingen andre objekter i scenen skal bli tegnet med tekstur
	    gl.glDisable(GL_TEXTURE_2D);
	}

	
	/**
	 * <code>drawWall</code>
	 *
	 * Metode for  tegne vegger. Tar en array av hjrnekoordinater og normal som argument	
	 *
	 * @param coo[][] a <code>float</code> value
	 * @param normal[] a <code>float</code> value
	 */
	public void drawWall(float coo[][], float normal[]){
	    gl.glBegin(GL_QUADS);
	      //Setter normal
	      gl.glNormal3f(normal[0],normal[1],normal[2]);
	      for (int i = 0; i < coo.length; i++) {
		  //Tegner vegg
		  gl.glVertex3f(coo[i][0],coo[i][1],coo[i][2]);	    
	      }
	    gl.glEnd();
	}

	
	/**
	 * <code>reshape</code>
	 *
	 * Kalles nr vindu reskaleres
	 *
	 * @param w an <code>int</code> value
	 * @param h an <code>int</code> value
	 */
	public void reshape(int w, int h){
	    gl.glViewport(0,0,w,h);
	    gl.glMatrixMode(GL_PROJECTION);
	    gl.glLoadIdentity();
	    glu.gluPerspective(35.0, (float) w/h, 1.0, 500.0);
	    gl.glMatrixMode(GL_MODELVIEW);
	}
    }
}
